import datetime
import unittest

import jpholiday


class TestYear2007(unittest.TestCase):
    def test_holiday(self):
        """
        2007年祝日
        """
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 1, 1)), '元日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 1, 8)), '成人の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 2, 11)), '建国記念の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 2, 12)), '建国記念の日 振替休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 3, 21)), '春分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 4, 29)), '昭和の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 4, 30)), '昭和の日 振替休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 5, 3)), '憲法記念日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 5, 4)), 'みどりの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 5, 5)), 'こどもの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 7, 16)), '海の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 9, 17)), '敬老の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 9, 23)), '秋分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 9, 24)), '秋分の日 振替休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 10, 8)), '体育の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 11, 3)), '文化の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 11, 23)), '勤労感謝の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 12, 23)), '天皇誕生日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2007, 12, 24)), '天皇誕生日 振替休日')

    def test_count_month(self):
        """
        2007年月祝日数
        """
        self.assertEqual(len(jpholiday.month_holidays(2007, 1)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2007, 2)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2007, 3)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2007, 4)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2007, 5)), 3)
        self.assertEqual(len(jpholiday.month_holidays(2007, 6)), 0)
        self.assertEqual(len(jpholiday.month_holidays(2007, 7)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2007, 8)), 0)
        self.assertEqual(len(jpholiday.month_holidays(2007, 9)), 3)
        self.assertEqual(len(jpholiday.month_holidays(2007, 10)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2007, 11)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2007, 12)), 2)

    def test_count_year(self):
        """
        2007年祝日数
        """
        self.assertEqual(len(jpholiday.year_holidays(2007)), 19)



import datetime
from typing import Optional, Union

from jpholiday.jpholiday import JPHoliday, OriginalHolidayCheckerInterface

new_api = JPHoliday()


def is_holiday_name(date: Union[datetime.date, datetime.datetime]) -> Optional[str]:
    """
    その日の祝日名を返します。
    """

    result = new_api.holidays(date)

    if len(result) == 0:
        return None

    return result[0].name


def is_holiday(date: Union[datetime.date, datetime.datetime]) -> bool:
    """
    その日が祝日かどうかを返します。
    """

    return new_api.is_holiday(date)


def year_holidays(year: int) -> list[tuple[datetime.date, str]]:
    """
    その年の祝日日、祝日名を返します。
    """

    result = new_api.year_holidays(year)
    return [it.to_tuple() for it in result]


def month_holidays(year: int, month: int) -> list[tuple[datetime.date, str]]:
    """
    その月の祝日日、祝日名を返します。
    """

    result = new_api.month_holidays(year, month)
    return [it.to_tuple() for it in result]


def between(
        start_date: Union[datetime.date, datetime.datetime],
        end_date: Union[datetime.date, datetime.datetime]
) -> list[tuple[datetime.date, str]]:
    """
    指定された期間の祝日日、祝日名を返します。
    """

    result = new_api.between(start_date, end_date)
    return [it.to_tuple() for it in result]


def register(checker: OriginalHolidayCheckerInterface):
    """
    独自の祝日チェッカーを登録します。
    """
    new_api.register(checker)


def unregister(checker: OriginalHolidayCheckerInterface):
    """
    独自の祝日チェッカーを登録解除します。
    """
    new_api.unregister(checker)



from abc import abstractmethod, ABC

from jpholiday.checker.interface import HolidayCheckerInterface


class CheckerRegistryInterface(ABC):
    @abstractmethod
    def checkers(self) -> list[HolidayCheckerInterface]:
        pass

    @abstractmethod
    def register(self, checker: HolidayCheckerInterface):
        pass

    @abstractmethod
    def unregister(self, checker: HolidayCheckerInterface):
        pass



import datetime
import unittest

import jpholiday


class TestYear2005(unittest.TestCase):
    def test_holiday(self):
        """
        2005年祝日
        """
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2005, 1, 1)), '元日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2005, 1, 10)), '成人の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2005, 2, 11)), '建国記念の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2005, 3, 20)), '春分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2005, 3, 21)), '春分の日 振替休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2005, 4, 29)), 'みどりの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2005, 5, 3)), '憲法記念日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2005, 5, 4)), '国民の休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2005, 5, 5)), 'こどもの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2005, 7, 18)), '海の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2005, 9, 19)), '敬老の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2005, 9, 23)), '秋分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2005, 10, 10)), '体育の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2005, 11, 3)), '文化の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2005, 11, 23)), '勤労感謝の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2005, 12, 23)), '天皇誕生日')

    def test_count_month(self):
        """
        2005年月祝日数
        """
        self.assertEqual(len(jpholiday.month_holidays(2005, 1)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2005, 2)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2005, 3)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2005, 4)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2005, 5)), 3)
        self.assertEqual(len(jpholiday.month_holidays(2005, 6)), 0)
        self.assertEqual(len(jpholiday.month_holidays(2005, 7)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2005, 8)), 0)
        self.assertEqual(len(jpholiday.month_holidays(2005, 9)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2005, 10)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2005, 11)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2005, 12)), 1)

    def test_count_year(self):
        """
        2005年祝日数
        """
        self.assertEqual(len(jpholiday.year_holidays(2005)), 16)



import datetime
import unittest

import jpholiday


class TestYear2022(unittest.TestCase):
    def test_holiday(self):
        """
        2022年祝日
        """
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2022, 1, 1)), '元日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2022, 1, 10)), '成人の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2022, 2, 11)), '建国記念の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2022, 2, 23)), '天皇誕生日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2022, 3, 21)), '春分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2022, 4, 29)), '昭和の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2022, 5, 3)), '憲法記念日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2022, 5, 4)), 'みどりの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2022, 5, 5)), 'こどもの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2022, 7, 18)), '海の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2022, 8, 11)), '山の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2022, 9, 19)), '敬老の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2022, 9, 23)), '秋分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2022, 10, 10)), 'スポーツの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2022, 11, 3)), '文化の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2022, 11, 23)), '勤労感謝の日')

    def test_count_month(self):
        """
        2022年月祝日数
        """
        self.assertEqual(len(jpholiday.month_holidays(2022, 1)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2022, 2)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2022, 3)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2022, 4)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2022, 5)), 3)
        self.assertEqual(len(jpholiday.month_holidays(2022, 6)), 0)
        self.assertEqual(len(jpholiday.month_holidays(2022, 7)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2022, 8)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2022, 9)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2022, 10)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2022, 11)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2022, 12)), 0)

    def test_count_year(self):
        """
        2022年祝日数
        """
        self.assertEqual(len(jpholiday.year_holidays(2022)), 16)



import datetime
import unittest

import jpholiday
from jpholiday.exception import JPHolidayTypeError


class TestException(unittest.TestCase):

    def test_is_holiday_name(self):
        """
        型チェック
        """
        with self.assertRaises(JPHolidayTypeError):
            jpholiday.is_holiday_name(21210101)

        with self.assertRaises(JPHolidayTypeError):
            jpholiday.is_holiday_name('2021-01-01')

    def test_is_holiday(self):
        """
        型チェック
        """
        with self.assertRaises(JPHolidayTypeError):
            jpholiday.is_holiday(21210101)

        with self.assertRaises(JPHolidayTypeError):
            jpholiday.is_holiday('2021-01-01')

    def test_between(self):
        """
        型チェック
        """
        with self.assertRaises(JPHolidayTypeError):
            jpholiday.between(21210101, 20210101)

        with self.assertRaises(JPHolidayTypeError):
            jpholiday.between(datetime.date(2021, 1, 1), 20210101)

        with self.assertRaises(JPHolidayTypeError):
            jpholiday.between(20210101, datetime.date(2021, 1, 1))

        with self.assertRaises(JPHolidayTypeError):
            jpholiday.between('2021-01-01', '2021-01-01')

        with self.assertRaises(JPHolidayTypeError):
            jpholiday.between(datetime.date(2021, 1, 1), '2021-01-01')

        with self.assertRaises(JPHolidayTypeError):
            jpholiday.between('2021-01-01', datetime.date(2021, 1, 1))



import datetime
from typing import Any, Union, Iterator

from jpholiday.cache.in_memory import HolidayInMemoryCache
from jpholiday.checker.interface import OriginalHolidayCheckerInterface
from jpholiday.exception import JPHolidayTypeError
from jpholiday.model.holiday import Holiday
from jpholiday.registy.registry import HolidayCheckerRegistry


class JPHoliday:
    def __init__(self):
        self._cache = HolidayInMemoryCache()
        self.registry = HolidayCheckerRegistry()

    def holidays(self, date: Union[datetime.date, datetime.datetime]) -> list[Holiday]:
        """
        その日の祝日名を返します。
        """

        date = self._to_date(date)

        cache = self._cache.get(date)
        if cache is not None:
            return cache

        holidays = []
        for holiday in self.registry.checkers():
            if holiday.is_holiday(date):
                holidays.append(Holiday(date, holiday.holiday_name(date)))

        self._cache.set(date, holidays)

        return holidays

    def is_holiday(self, date: Union[datetime.date, datetime.datetime]) -> bool:
        """
        その日が祝日かどうかを返します。
        """

        date = self._to_date(date)

        if len(self.holidays(date)) != 0:
            return True

        return False

    def year_holidays(self, year: int) -> list[Holiday]:
        """
        その年の祝日日、祝日名を返します。
        """
        return list(self.iter_year_holidays(year))

    def iter_year_holidays(self, year: int) -> Iterator[Holiday]:
        """
        指定された年の祝日日、祝日名をイテレーターで返します。
        """
        date = datetime.date(year, 1, 1)

        while date.year == year:
            result = self.holidays(date)
            if result:
                for holiday in result:
                    yield holiday

            date = date + datetime.timedelta(days=1)

    def month_holidays(self, year: int, month: int) -> list[Holiday]:
        """
        その月の祝日日、祝日名を返します。
        """
        return list(self.iter_month_holidays(year, month))

    def iter_month_holidays(self, year: int, month: int) -> Iterator[Holiday]:
        """
        指定された月の祝日日、祝日名をイテレーターで返します。
        """
        date = datetime.date(year, month, 1)

        while date.month == month:
            result = self.holidays(date)
            if result:
                for holiday in result:
                    yield holiday

            date = date + datetime.timedelta(days=1)

    def between(
            self,
            start_date: Union[datetime.date, datetime.datetime],
            end_date: Union[datetime.date, datetime.datetime]
    ) -> list[Holiday]:
        """
        指定された期間の祝日日、祝日名をリストで返します。
        """
        return list(self.iter_between(start_date, end_date))

    def iter_between(
            self,
            start_date: Union[datetime.date, datetime.datetime],
            end_date: Union[datetime.date, datetime.datetime]
    ) -> Iterator[Holiday]:
        """
        指定された期間の祝日日、祝日名をイテレーターで返します。
        """
        current_date = self._to_date(start_date)
        end_date = self._to_date(end_date)

        while current_date <= end_date:
            result = self.holidays(current_date)
            if result:
                for holiday in result:
                    yield holiday

            current_date = current_date + datetime.timedelta(days=1)

    def _to_date(self, value: Any) -> datetime.date:
        """
        datetime型をdate型へ変換
        それ以外は例外
        """
        if isinstance(value, datetime.datetime):
            return value.date()
        if isinstance(value, datetime.date):
            return value
        raise JPHolidayTypeError("is type datetime or date isinstance only.")

    def register(self, checker: OriginalHolidayCheckerInterface):
        """
        独自の祝日チェッカーを登録します。
        """
        self._cache.clear()
        self.registry.register(checker)

    def unregister(self, checker: OriginalHolidayCheckerInterface):
        """
        独自の祝日チェッカーを登録解除します。
        """
        self._cache.clear()
        self.registry.unregister(checker)



from jpholiday.checker.checker import *
from jpholiday.checker.interface import HolidayCheckerInterface
from jpholiday.registy.interface import CheckerRegistryInterface


class HolidayCheckerRegistry(CheckerRegistryInterface):
    def __init__(self):
        self._checker: list[HolidayCheckerInterface] = [
            NewYearChecker(),
            AdultDayChecker(),
            FoundationDayChecker(),
            EmperorsBirthdayChecker(),
            VernalEquinoxDayChecker(),
            GreeneryDayChecker(),
            ShowaDayChecker(),
            ConstitutionMemorialDayChecker(),
            ChildrensDayChecker(),
            SeaDayChecker(),
            MountainDayChecker(),
            RespectForTheAgedDayChecker(),
            AutumnEquinoxDayChecker(),
            HealthAndSportsDayChecker(),
            SportsDayChecker(),
            CultureDayChecker(),
            LaborThanksgivingDayChecker(),
            ExtraHoliday1959Checker(),
            ExtraHoliday1989Checker(),
            ExtraHoliday1990Checker(),
            ExtraHoliday1993Checker(),
            ExtraHoliday2019MayChecker(),
            ExtraHoliday2019OctChecker(),
            TransferHolidayChecker(self),
            NationalHolidayChecker(self),
        ]

    def checkers(self) -> list[HolidayCheckerInterface]:
        return self._checker

    def register(self, checker: HolidayCheckerInterface):
        if any(isinstance(h, type(checker)) for h in self._checker):
            return
        self._checker.append(checker)

    def unregister(self, checker: HolidayCheckerInterface):
        self._checker = [h for h in self._checker if not isinstance(h, type(checker))]



import datetime
import unittest

import jpholiday


class TestYear2006(unittest.TestCase):
    def test_holiday(self):
        """
        2006年祝日
        """
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2006, 1, 1)), '元日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2006, 1, 2)), '元日 振替休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2006, 1, 9)), '成人の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2006, 2, 11)), '建国記念の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2006, 3, 21)), '春分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2006, 4, 29)), 'みどりの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2006, 5, 3)), '憲法記念日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2006, 5, 4)), '国民の休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2006, 5, 5)), 'こどもの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2006, 7, 17)), '海の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2006, 9, 18)), '敬老の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2006, 9, 23)), '秋分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2006, 10, 9)), '体育の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2006, 11, 3)), '文化の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2006, 11, 23)), '勤労感謝の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2006, 12, 23)), '天皇誕生日')

    def test_count_month(self):
        """
        2006年月祝日数
        """
        self.assertEqual(len(jpholiday.month_holidays(2006, 1)), 3)
        self.assertEqual(len(jpholiday.month_holidays(2006, 2)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2006, 3)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2006, 4)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2006, 5)), 3)
        self.assertEqual(len(jpholiday.month_holidays(2006, 6)), 0)
        self.assertEqual(len(jpholiday.month_holidays(2006, 7)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2006, 8)), 0)
        self.assertEqual(len(jpholiday.month_holidays(2006, 9)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2006, 10)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2006, 11)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2006, 12)), 1)

    def test_count_year(self):
        """
        2006年祝日数
        """
        self.assertEqual(len(jpholiday.year_holidays(2006)), 16)



import datetime
import unittest

from jpholiday import JPHoliday
from jpholiday.model.holiday import Holiday


class TestClass(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def test_main(self):
        jpholiday = JPHoliday()

        self.assertEqual(
            jpholiday.holidays(datetime.date(2020, 1, 1)),
            [
                Holiday(
                    date=datetime.date(2020, 1, 1),
                    name='元日',
                ),
            ]
        )
        self.assertEqual(jpholiday.holidays(datetime.date(2020, 2, 3)), [])
        self.assertEqual(jpholiday.is_holiday(datetime.date(2020, 2, 3)), False)



import datetime
import math

from jpholiday.checker import utils
from jpholiday.checker.interface import HolidayCheckerInterface, OriginalHolidayCheckerInterface
from jpholiday.registy.interface import CheckerRegistryInterface


# 元日
class NewYearChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        return date.month == 1 and date.day == 1

    def holiday_name(self, date: datetime.date) -> str:
        return '元日'


# 成人の日
class AdultDayChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        if date.year <= 1999 and date.month == 1 and date.day == 15:
            return True
        elif date.year >= 2000 and date.month == 1 and date.day == utils._week_day(date, 2, 1).day:
            return True
        return False

    def holiday_name(self, date: datetime.date) -> str:
        return '成人の日'


# 建国記念の日
class FoundationDayChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        return date.year >= 1967 and date.month == 2 and date.day == 11

    def holiday_name(self, date: datetime.date) -> str:
        return '建国記念の日'


# 天皇誕生日
class EmperorsBirthdayChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        if date.year in range(1948, 1988 + 1) and date.month == 4 and date.day == 29:
            return True
        elif date.year in range(1989, 2018 + 1) and date.month == 12 and date.day == 23:
            return True
        elif date.year >= 2020 and date.month == 2 and date.day == 23:
            return True
        return False

    def holiday_name(self, date: datetime.date) -> str:
        return '天皇誕生日'


# 春分の日
class VernalEquinoxDayChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        if date.month == 3 and date.day == self._vernal_equinox_day(date.year):
            return True
        return False

    def holiday_name(self, date: datetime.date) -> str:
        return '春分の日'

    @staticmethod
    def _vernal_equinox_day(year):
        """
        春季皇霊祭: 1879-1947
        春分の日: 1948
        春分の日の日付を返します。
        http://mt-soft.sakura.ne.jp/kyozai/excel_high/200_jissen_kiso/60_syunbun.htm
        """

        if year <= 1948:
            return 0

        if year >= 1851 and year <= 1899:
            i = 19.8277
        elif year >= 1900 and year <= 1979:
            i = 20.8357
        elif year >= 1980 and year <= 2099:
            i = 20.8431
        elif year >= 2100 and year <= 2150:
            i = 21.8510
        else:
            i = 0

        return math.floor(i + 0.242194 * (year - 1980) - math.floor((year - 1980) / 4))


# みどりの日
class GreeneryDayChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        if date.year in range(1989, 2006 + 1) and date.month == 4 and date.day == 29:
            return True
        elif date.year >= 2007 and date.month == 5 and date.day == 4:
            return True
        return False

    def holiday_name(self, date: datetime.date) -> str:
        return 'みどりの日'


# 昭和の日
class ShowaDayChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        if date.year >= 2007 and date.month == 4 and date.day == 29:
            return True
        return False

    def holiday_name(self, date: datetime.date) -> str:
        return '昭和の日'


# 憲法記念日
class ConstitutionMemorialDayChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        if date.month == 5 and date.day == 3:
            return True
        return False

    def holiday_name(self, date: datetime.date) -> str:
        return '憲法記念日'


# こどもの日
class ChildrensDayChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        if date.month == 5 and date.day == 5:
            return True
        return False

    def holiday_name(self, date: datetime.date) -> str:
        return 'こどもの日'


# 海の日
class SeaDayChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        # 2020: 国民の祝日に関する法律(昭和23年法律第178号)の特例
        if date.year == 2020:
            if date == datetime.date(2020, 7, 23):
                return True

            return False

        # 2021: 五輪特別措置法改正案
        if date.year == 2021:
            if date == datetime.date(2021, 7, 22):
                return True

            return False

        if date.year in range(1996, 2002 + 1) and date.month == 7 and date.day == 20:
            return True
        # 2020: 国民の祝日に関する法律の一部を改正する法律(平成30年法律第57号)
        elif date.year >= 2003 and date.month == 7 and date.day == utils._week_day(date, 3,
                                                                                   1).day:
            return True

        return False

    def holiday_name(self, date: datetime.date) -> str:
        return '海の日'


# 山の日
class MountainDayChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        # 2020: 国民の祝日に関する法律(昭和23年法律第178号)の特例
        if date.year == 2020:
            if date == datetime.date(2020, 8, 10):
                return True

            return False

        # 2021: 五輪特別措置法改正案
        if date.year == 2021:
            if date == datetime.date(2021, 8, 8):
                return True

            return False

        # 2016: 国民の祝日に関する法律の一部を改正する法律(平成26年法律第43号)
        # 2020: 国民の祝日に関する法律の一部を改正する法律(平成30年法律第57号)
        if date.year >= 2016 and date.month == 8 and date.day == 11:
            return True

        return False

    def holiday_name(self, date: datetime.date) -> str:
        return '山の日'


# 敬老の日
class RespectForTheAgedDayChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        if date.year in range(1966, 2002 + 1) and date.month == 9 and date.day == 15:
            return True
        elif date.year >= 2003 and date.month == 9 and date.day == utils._week_day(date, 3, 1).day:
            return True
        return False

    def holiday_name(self, date: datetime.date) -> str:
        return '敬老の日'


# 秋分の日
class AutumnEquinoxDayChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        if date.month == 9 and date.day == self._autumn_equinox_day(date.year):
            return True
        return False

    def holiday_name(self, date: datetime.date) -> str:
        return '秋分の日'

    @staticmethod
    def _autumn_equinox_day(year):
        """
        秋分の日の日付を返します。
        秋季皇霊祭: 1879-1947
        秋分の日: 1948
        http://mt-soft.sakura.ne.jp/kyozai/excel_high/200_jissen_kiso/60_syunbun.htm
        """

        if year <= 1948:
            return 0

        if year >= 1851 and year <= 1899:
            i = 22.2588
        elif year >= 1900 and year <= 1979:
            i = 23.2588
        elif year >= 1980 and year <= 2099:
            i = 23.2488
        elif year >= 2100 and year <= 2150:
            i = 24.2488
        else:
            i = 0

        return math.floor(i + 0.242194 * (year - 1980) - math.floor((year - 1980) / 4))


# 体育の日
class HealthAndSportsDayChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        if date.year in range(1966, 1999 + 1) and date.month == 10 and date.day == 10:
            return True
        elif date.year in range(2000, 2019 + 1) and date.month == 10 and date.day == utils._week_day(date, 2, 1).day:
            return True
        return False

    def holiday_name(self, date: datetime.date) -> str:
        return '体育の日'


# スポーツの日
class SportsDayChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        # 2020: 国民の祝日に関する法律(昭和23年法律第178号)の特例
        if date.year == 2020:
            if date == datetime.date(2020, 7, 24):
                return True

            return False

        # 2021: 五輪特別措置法改正案
        if date.year == 2021:
            if date == datetime.date(2021, 7, 23):
                return True

            return False

        # 2020: 国民の祝日に関する法律の一部を改正する法律(平成30年法律第57号)
        #       国民の祝日に関する法律(昭和23年法律第178号)の特例
        if date.year >= 2020 and date.month == 10 and date.day == utils._week_day(date, 2, 1).day:
            return True

        return False

    def holiday_name(self, date: datetime.date) -> str:
        return 'スポーツの日'


# 文化の日
class CultureDayChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        if date.month == 11 and date.day == 3:
            return True
        return False

    def holiday_name(self, date: datetime.date) -> str:
        return '文化の日'


# 勤労感謝の日
class LaborThanksgivingDayChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        if date.month == 11 and date.day == 23:
            return True
        return False

    def holiday_name(self, date: datetime.date) -> str:
        return '勤労感謝の日'


# 1959年 皇太子・明仁親王の結婚の儀
class ExtraHoliday1959Checker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        return date == datetime.date(1959, 4, 10)

    def holiday_name(self, date: datetime.date) -> str:
        return '皇太子・明仁親王の結婚の儀'


# 1989年 昭和天皇の大喪の礼
class ExtraHoliday1989Checker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        return date == datetime.date(1989, 2, 24)

    def holiday_name(self, date: datetime.date) -> str:
        return '昭和天皇の大喪の礼'


# 1990年 即位の礼正殿の儀
class ExtraHoliday1990Checker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        return date == datetime.date(1990, 11, 12)

    def holiday_name(self, date: datetime.date) -> str:
        return '即位の礼正殿の儀'


# 1993年 皇太子・皇太子徳仁親王の結婚の儀
class ExtraHoliday1993Checker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        return date == datetime.date(1993, 6, 9)

    def holiday_name(self, date: datetime.date) -> str:
        return '皇太子・皇太子徳仁親王の結婚の儀'


# 2019年5月1日 天皇の即位の日
class ExtraHoliday2019MayChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        return date == datetime.date(2019, 5, 1)

    def holiday_name(self, date: datetime.date) -> str:
        return '天皇の即位の日'


# 2019年10月22日 即位礼正殿の儀
class ExtraHoliday2019OctChecker(HolidayCheckerInterface):
    def is_holiday(self, date: datetime.date) -> bool:
        return date == datetime.date(2019, 10, 22)

    def holiday_name(self, date: datetime.date) -> str:
        return '即位礼正殿の儀'


# 振替休日
class TransferHolidayChecker(HolidayCheckerInterface):

    def __init__(self, registry: CheckerRegistryInterface) -> None:
        self._registry = registry

    def is_holiday(self, date: datetime.date) -> bool:
        if self.__transfer_holiday_name(date) == "":
            return False
        return True

    def holiday_name(self, date: datetime.date) -> str:
        return self.__transfer_holiday_name(date)

    def __transfer_holiday_name(self, date: datetime.date) -> str:
        # 1973年(昭和48年)4月12日 - 改正・施行
        if date.year < 1973:
            return ""

        # 日曜日に振替休日は存在しない
        if date.isoweekday() == 7:
            return ""

        filtered_registry = list(
            filter(
                lambda x:
                not isinstance(x, TransferHolidayChecker)
                and not isinstance(x, NationalHolidayChecker)
                and not isinstance(x, OriginalHolidayCheckerInterface),
                self._registry.checkers()
            )
        )

        # 祝日が存在する日に振替休日は存在しない
        if len(list(filter(lambda x: x.is_holiday(date), filtered_registry))) != 0:
            return ""

        current_date = date - datetime.timedelta(days=1)

        while (True):
            current_registry = list(filter(lambda x: x.is_holiday(current_date), filtered_registry))
            if len(current_registry) == 0:
                return ""

            if current_date.isoweekday() == 7:
                if len(current_registry) != 0:
                    return '{} {}'.format(
                        current_registry[0].holiday_name(current_date),
                        '振替休日'
                    )
                else:
                    return ""

            current_date = current_date - datetime.timedelta(days=1)


# 国民の休日
class NationalHolidayChecker(HolidayCheckerInterface):
    def __init__(self, registry: CheckerRegistryInterface) -> None:
        self._registry = registry

    def is_holiday(self, date: datetime.date) -> bool:

        if date.isoweekday() == 7:
            return False

        filtered_registry = list(
            filter(
                lambda x:
                not isinstance(x, NationalHolidayChecker)
                and not isinstance(x, OriginalHolidayCheckerInterface),
                self._registry.checkers()
            )
        )

        if len(list(filter(lambda x: x.is_holiday(date), filtered_registry))) != 0:
            return False

        if (
                len(list(filter(lambda x: x.is_holiday(date + datetime.timedelta(days=1)), filtered_registry))) != 0
                and len(list(filter(lambda x: x.is_holiday(date - datetime.timedelta(days=1)), filtered_registry))) != 0
        ):
            return True

        return False

    def holiday_name(self, date: datetime.date) -> str:
        return '国民の休日'



import datetime
import unittest

import jpholiday


class TestYear2012(unittest.TestCase):
    def test_holiday(self):
        """
        2012年祝日
        """
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2012, 1, 1)), '元日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2012, 1, 2)), '元日 振替休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2012, 1, 9)), '成人の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2012, 2, 11)), '建国記念の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2012, 3, 20)), '春分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2012, 4, 29)), '昭和の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2012, 4, 30)), '昭和の日 振替休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2012, 5, 3)), '憲法記念日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2012, 5, 4)), 'みどりの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2012, 5, 5)), 'こどもの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2012, 7, 16)), '海の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2012, 9, 17)), '敬老の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2012, 9, 22)), '秋分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2012, 10, 8)), '体育の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2012, 11, 3)), '文化の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2012, 11, 23)), '勤労感謝の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2012, 12, 23)), '天皇誕生日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2012, 12, 24)), '天皇誕生日 振替休日')

    def test_count_month(self):
        """
        2012年月祝日数
        """
        self.assertEqual(len(jpholiday.month_holidays(2012, 1)), 3)
        self.assertEqual(len(jpholiday.month_holidays(2012, 2)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2012, 3)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2012, 4)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2012, 5)), 3)
        self.assertEqual(len(jpholiday.month_holidays(2012, 6)), 0)
        self.assertEqual(len(jpholiday.month_holidays(2012, 7)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2012, 8)), 0)
        self.assertEqual(len(jpholiday.month_holidays(2012, 9)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2012, 10)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2012, 11)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2012, 12)), 2)

    def test_count_year(self):
        """
        2012年祝日数
        """
        self.assertEqual(len(jpholiday.year_holidays(2012)), 18)



import datetime
import unittest

import jpholiday


class TestDateTime(unittest.TestCase):

    def test_datetime_sea_day(self):
        """
        海の日
        """
        self.assertEqual(jpholiday.is_holiday_name(datetime.datetime(2020, 7, 23, 1, 1, 1)), '海の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.datetime(2021, 7, 22, 1, 1, 1)), '海の日')
        self.assertTrue(jpholiday.is_holiday(datetime.datetime(2020, 7, 23, 1, 1, 1)))
        self.assertTrue(jpholiday.is_holiday(datetime.datetime(2021, 7, 22, 1, 1, 1)))
        self.assertEqual(
            jpholiday.between(datetime.datetime(2020, 7, 23, 1, 1, 1), datetime.datetime(2020, 7, 23, 1, 1, 1)),
            [(datetime.date(2020, 7, 23), '海の日')]
        )
        self.assertEqual(
            jpholiday.between(datetime.datetime(2021, 7, 22, 1, 1, 1), datetime.datetime(2021, 7, 22, 1, 1, 1)),
            [(datetime.date(2021, 7, 22), '海の日')]
        )

    def test_datetime_mountain_day(self):
        """
        山の日
        """
        self.assertEqual(jpholiday.is_holiday_name(datetime.datetime(2020, 8, 10, 1, 1, 1)), '山の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.datetime(2021, 8, 8, 1, 1, 1)), '山の日')
        self.assertTrue(jpholiday.is_holiday(datetime.datetime(2020, 8, 10, 1, 1, 1)))
        self.assertTrue(jpholiday.is_holiday(datetime.datetime(2021, 8, 8, 1, 1, 1)))
        self.assertEqual(
            jpholiday.between(datetime.datetime(2020, 8, 10, 1, 1, 1), datetime.datetime(2020, 8, 10, 1, 1, 1)),
            [(datetime.date(2020, 8, 10), '山の日')]
        )
        self.assertEqual(
            jpholiday.between(datetime.datetime(2021, 8, 8, 1, 1, 1), datetime.datetime(2021, 8, 8, 1, 1, 1)),
            [(datetime.date(2021, 8, 8), '山の日')]
        )

    def test_datetime_sports_day(self):
        """
        スポーツの日
        """
        self.assertEqual(jpholiday.is_holiday_name(datetime.datetime(2020, 7, 24, 1, 1, 1)), 'スポーツの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.datetime(2021, 7, 23, 1, 1, 1)), 'スポーツの日')
        self.assertTrue(jpholiday.is_holiday_name(datetime.datetime(2020, 7, 24, 1, 1, 1)))
        self.assertTrue(jpholiday.is_holiday_name(datetime.datetime(2021, 7, 23, 1, 1, 1)))
        self.assertEqual(
            jpholiday.between(datetime.datetime(2020, 7, 24, 1, 1, 1), datetime.datetime(2020, 7, 24, 1, 1, 1)),
            [(datetime.date(2020, 7, 24), 'スポーツの日')]
        )
        self.assertEqual(
            jpholiday.between(datetime.datetime(2021, 7, 23, 1, 1, 1), datetime.datetime(2021, 7, 23, 1, 1, 1)),
            [(datetime.date(2021, 7, 23), 'スポーツの日')]
        )

    def test_datetime_other_holiday(self):
        """
        皇室慶弔行事に伴う祝日
        """
        self.assertEqual(jpholiday.is_holiday_name(datetime.datetime(1959, 4, 10, 1, 1, 1)),
                         '皇太子・明仁親王の結婚の儀')
        self.assertEqual(jpholiday.is_holiday_name(datetime.datetime(1989, 2, 24, 1, 1, 1)), '昭和天皇の大喪の礼')
        self.assertEqual(jpholiday.is_holiday_name(datetime.datetime(1990, 11, 12, 1, 1, 1)), '即位の礼正殿の儀')
        self.assertEqual(jpholiday.is_holiday_name(datetime.datetime(1993, 6, 9, 1, 1, 1)),
                         '皇太子・皇太子徳仁親王の結婚の儀')
        self.assertEqual(jpholiday.is_holiday_name(datetime.datetime(2019, 5, 1, 1, 1, 1)), '天皇の即位の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.datetime(2019, 10, 22, 1, 1, 1)), '即位礼正殿の儀')
        self.assertTrue(jpholiday.is_holiday(datetime.datetime(1959, 4, 10, 1, 1, 1)))
        self.assertTrue(jpholiday.is_holiday(datetime.datetime(1989, 2, 24, 1, 1, 1)))
        self.assertTrue(jpholiday.is_holiday(datetime.datetime(1990, 11, 12, 1, 1, 1)))
        self.assertTrue(jpholiday.is_holiday(datetime.datetime(1993, 6, 9, 1, 1, 1)))
        self.assertTrue(jpholiday.is_holiday(datetime.datetime(2019, 5, 1, 1, 1, 1)))
        self.assertTrue(jpholiday.is_holiday(datetime.datetime(2019, 10, 22, 1, 1, 1)))
        self.assertEqual(
            jpholiday.between(datetime.datetime(1959, 4, 10, 1, 1, 1), datetime.datetime(1959, 4, 10, 1, 1, 1)),
            [(datetime.date(1959, 4, 10), '皇太子・明仁親王の結婚の儀')]
        )
        self.assertEqual(
            jpholiday.between(datetime.datetime(1989, 2, 24, 1, 1, 1), datetime.datetime(1989, 2, 24, 1, 1, 1)),
            [(datetime.date(1989, 2, 24), '昭和天皇の大喪の礼')]
        )
        self.assertEqual(
            jpholiday.between(datetime.datetime(1990, 11, 12, 1, 1, 1), datetime.datetime(1990, 11, 12, 1, 1, 1)),
            [(datetime.date(1990, 11, 12), '即位の礼正殿の儀')]
        )
        self.assertEqual(
            jpholiday.between(datetime.datetime(1993, 6, 9, 1, 1, 1), datetime.datetime(1993, 6, 9, 1, 1, 1)),
            [(datetime.date(1993, 6, 9), '皇太子・皇太子徳仁親王の結婚の儀')]
        )
        self.assertEqual(
            jpholiday.between(datetime.datetime(2019, 5, 1, 1, 1, 1), datetime.datetime(2019, 5, 1, 1, 1, 1)),
            [(datetime.date(2019, 5, 1), '天皇の即位の日')]
        )
        self.assertEqual(
            jpholiday.between(datetime.datetime(2019, 10, 22, 1, 1, 1), datetime.datetime(2019, 10, 22, 1, 1, 1)),
            [(datetime.date(2019, 10, 22), '即位礼正殿の儀')]
        )



import datetime
from abc import abstractmethod, ABC


class HolidayCheckerInterface(ABC):
    @abstractmethod
    def is_holiday(self, date: datetime.date) -> bool:
        pass

    @abstractmethod
    def holiday_name(self, date: datetime.date) -> str:
        pass


class OriginalHolidayCheckerInterface(HolidayCheckerInterface, ABC):
    pass



import datetime
import unittest

import jpholiday


class TestYear2001(unittest.TestCase):
    def test_holiday(self):
        """
        2001年祝日
        """
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 1, 1)), '元日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 1, 8)), '成人の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 2, 11)), '建国記念の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 2, 12)), '建国記念の日 振替休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 3, 20)), '春分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 4, 29)), 'みどりの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 4, 30)), 'みどりの日 振替休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 5, 3)), '憲法記念日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 5, 4)), '国民の休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 5, 5)), 'こどもの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 7, 20)), '海の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 9, 15)), '敬老の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 9, 23)), '秋分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 9, 24)), '秋分の日 振替休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 10, 8)), '体育の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 11, 3)), '文化の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 11, 23)), '勤労感謝の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 12, 23)), '天皇誕生日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2001, 12, 24)), '天皇誕生日 振替休日')

    def test_count_month(self):
        """
        2001年月祝日数
        """
        self.assertEqual(len(jpholiday.month_holidays(2001, 1)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2001, 2)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2001, 3)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2001, 4)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2001, 5)), 3)
        self.assertEqual(len(jpholiday.month_holidays(2001, 6)), 0)
        self.assertEqual(len(jpholiday.month_holidays(2001, 7)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2001, 8)), 0)
        self.assertEqual(len(jpholiday.month_holidays(2001, 9)), 3)
        self.assertEqual(len(jpholiday.month_holidays(2001, 10)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2001, 11)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2001, 12)), 2)

    def test_count_year(self):
        """
        2001年祝日数
        """
        self.assertEqual(len(jpholiday.year_holidays(2001)), 19)



import datetime
from typing import Optional

from jpholiday.cache.interface import HolidayCacheInterface
from jpholiday.model.holiday import Holiday


class HolidayInMemoryCache(HolidayCacheInterface):
    def __init__(self):
        self._cache: dict[datetime.date, list[Holiday]] = {}

    def get(self, date: datetime.date) -> Optional[list[Holiday]]:
        return self._cache.get(date, None)

    def set(self, date: datetime.date, holidays: list[Holiday]):
        self._cache[date] = holidays

    def delete(self, date: datetime.date):
        self._cache.pop(date, None)

    def clear(self):
        self._cache.clear()



import datetime
import unittest

import jpholiday


class TestYear2016(unittest.TestCase):
    def test_holiday(self):
        """
        2016年祝日
        """
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2016, 1, 1)), '元日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2016, 1, 11)), '成人の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2016, 2, 11)), '建国記念の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2016, 3, 20)), '春分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2016, 3, 21)), '春分の日 振替休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2016, 4, 29)), '昭和の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2016, 5, 3)), '憲法記念日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2016, 5, 4)), 'みどりの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2016, 5, 5)), 'こどもの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2016, 7, 18)), '海の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2016, 8, 11)), '山の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2016, 9, 19)), '敬老の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2016, 9, 22)), '秋分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2016, 10, 10)), '体育の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2016, 11, 3)), '文化の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2016, 11, 23)), '勤労感謝の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2016, 12, 23)), '天皇誕生日')

    def test_count_month(self):
        """
        2016年月祝日数
        """
        self.assertEqual(len(jpholiday.month_holidays(2016, 1)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2016, 2)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2016, 3)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2016, 4)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2016, 5)), 3)
        self.assertEqual(len(jpholiday.month_holidays(2016, 6)), 0)
        self.assertEqual(len(jpholiday.month_holidays(2016, 7)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2016, 8)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2016, 9)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2016, 10)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2016, 11)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2016, 12)), 1)

    def test_count_year(self):
        """
        2016年祝日数
        """
        self.assertEqual(len(jpholiday.year_holidays(2016)), 17)



import datetime
import unittest

import jpholiday


class TestYear2000(unittest.TestCase):
    def test_holiday(self):
        """
        2000年祝日
        """
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2000, 1, 1)), '元日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2000, 1, 10)), '成人の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2000, 2, 11)), '建国記念の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2000, 3, 20)), '春分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2000, 4, 29)), 'みどりの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2000, 5, 3)), '憲法記念日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2000, 5, 4)), '国民の休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2000, 5, 5)), 'こどもの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2000, 7, 20)), '海の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2000, 9, 15)), '敬老の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2000, 9, 23)), '秋分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2000, 10, 9)), '体育の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2000, 11, 3)), '文化の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2000, 11, 23)), '勤労感謝の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2000, 12, 23)), '天皇誕生日')

    def test_count_month(self):
        """
        2000年月祝日数
        """
        self.assertEqual(len(jpholiday.month_holidays(2000, 1)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2000, 2)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2000, 3)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2000, 4)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2000, 5)), 3)
        self.assertEqual(len(jpholiday.month_holidays(2000, 6)), 0)
        self.assertEqual(len(jpholiday.month_holidays(2000, 7)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2000, 8)), 0)
        self.assertEqual(len(jpholiday.month_holidays(2000, 9)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2000, 10)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2000, 11)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2000, 12)), 1)

    def test_count_year(self):
        """
        2000年祝日数
        """
        self.assertEqual(len(jpholiday.year_holidays(2000)), 15)



import dataclasses
import datetime


@dataclasses.dataclass(frozen=True)
class Holiday:
    date: datetime.date
    name: str

    def to_tuple(self) -> tuple[datetime.date, str]:
        return self.date, self.name



import datetime
from abc import ABC, abstractmethod

from jpholiday.model.holiday import Holiday


class HolidayCacheInterface(ABC):
    @abstractmethod
    def get(self, date: datetime.date) -> list[Holiday]:
        pass

    @abstractmethod
    def set(self, date: datetime.date, holidays: list[Holiday]):
        pass

    @abstractmethod
    def delete(self, date: datetime.date):
        pass

    @abstractmethod
    def clear(self):
        pass



import datetime
import unittest

import jpholiday


class TestBasic(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def test_basic(self):
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2020, 2, 3)), None)
        self.assertEqual(jpholiday.is_holiday(datetime.date(2020, 2, 3)), False)
        
    def test_original_holiday(self):
        """
        独自の休み
        """

        class TestHoliday(jpholiday.OriginalHolidayCheckerInterface):
            def is_holiday(self, date: datetime.date) -> bool:
                if date == datetime.date(2020, 2, 3) or date == datetime.date(2020, 2, 5):
                    return True
                if date == datetime.date(2020, 2, 9):
                    return True
                return False

            def holiday_name(self, date: datetime.date) -> str:
                return '特別休暇'

        jpholiday.register(TestHoliday())

        # 国民の休日
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2020, 2, 3)), '特別休暇')
        self.assertEqual(jpholiday.is_holiday(datetime.date(2020, 2, 4)), False)
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2020, 2, 5)), '特別休暇')

        # 振替休日
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2020, 2, 9)), '特別休暇')
        self.assertEqual(jpholiday.is_holiday(datetime.date(2020, 2, 10)), False)

        # 登録解除
        jpholiday.unregister(TestHoliday())

        # 再登録
        jpholiday.register(TestHoliday())

        # 国民の休日
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2020, 2, 3)), '特別休暇')
        self.assertEqual(jpholiday.is_holiday(datetime.date(2020, 2, 4)), False)
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2020, 2, 5)), '特別休暇')

        # 振替休日
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2020, 2, 9)), '特別休暇')
        self.assertEqual(jpholiday.is_holiday(datetime.date(2020, 2, 10)), False)

        # 登録解除
        jpholiday.unregister(TestHoliday())

    def test_vernal_equinox_day(self):
        """
        春分の日
        """
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2000), 20)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2001), 20)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2002), 21)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2003), 21)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2004), 20)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2005), 20)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2006), 21)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2007), 21)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2008), 20)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2009), 20)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2010), 21)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2011), 21)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2012), 20)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2013), 20)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2014), 21)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2015), 21)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2016), 20)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2017), 20)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2018), 21)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2019), 21)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2020), 20)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2021), 20)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2022), 21)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2023), 21)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2024), 20)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2025), 20)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2026), 20)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2027), 21)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2028), 20)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2029), 20)
        self.assertEqual(jpholiday.checker.checker.VernalEquinoxDayChecker._vernal_equinox_day(2030), 20)

    def test_autumn_equinox_day(self):
        """
        秋分の日
        """
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2000), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2001), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2002), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2003), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2004), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2005), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2006), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2007), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2008), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2009), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2010), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2011), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2012), 22)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2013), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2014), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2015), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2016), 22)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2017), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2018), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2019), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2020), 22)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2021), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2022), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2023), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2024), 22)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2025), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2026), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2027), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2028), 22)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2029), 23)
        self.assertEqual(jpholiday.checker.checker.AutumnEquinoxDayChecker._autumn_equinox_day(2030), 23)

    def test_other_holiday(self):
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1959, 4, 10)), '皇太子・明仁親王の結婚の儀')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1989, 2, 24)), '昭和天皇の大喪の礼')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1990, 11, 12)), '即位の礼正殿の儀')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1993, 6, 9)), '皇太子・皇太子徳仁親王の結婚の儀')



import datetime
import unittest

import jpholiday


class TestYear1998(unittest.TestCase):
    def test_holiday(self):
        """
        1998年祝日
        """
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1998, 1, 1)), '元日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1998, 1, 15)), '成人の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1998, 2, 11)), '建国記念の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1998, 3, 21)), '春分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1998, 4, 29)), 'みどりの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1998, 5, 3)), '憲法記念日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1998, 5, 4)), '憲法記念日 振替休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1998, 5, 5)), 'こどもの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1998, 5, 6)), None)
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1998, 7, 20)), '海の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1998, 9, 15)), '敬老の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1998, 9, 23)), '秋分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1998, 10, 10)), '体育の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1998, 11, 3)), '文化の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1998, 11, 23)), '勤労感謝の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(1998, 12, 23)), '天皇誕生日')

    def test_count_month(self):
        """
        1998年月祝日数
        """
        self.assertEqual(len(jpholiday.month_holidays(1998, 1)), 2)
        self.assertEqual(len(jpholiday.month_holidays(1998, 2)), 1)
        self.assertEqual(len(jpholiday.month_holidays(1998, 3)), 1)
        self.assertEqual(len(jpholiday.month_holidays(1998, 4)), 1)
        self.assertEqual(len(jpholiday.month_holidays(1998, 5)), 3)
        self.assertEqual(len(jpholiday.month_holidays(1998, 6)), 0)
        self.assertEqual(len(jpholiday.month_holidays(1998, 7)), 1)
        self.assertEqual(len(jpholiday.month_holidays(1998, 8)), 0)
        self.assertEqual(len(jpholiday.month_holidays(1998, 9)), 2)
        self.assertEqual(len(jpholiday.month_holidays(1998, 10)), 1)
        self.assertEqual(len(jpholiday.month_holidays(1998, 11)), 2)
        self.assertEqual(len(jpholiday.month_holidays(1998, 12)), 1)

    def test_count_year(self):
        """
        1998年祝日数
        """
        self.assertEqual(len(jpholiday.year_holidays(1998)), 15)



import datetime
import unittest

import jpholiday


class TestYear2024(unittest.TestCase):
    def test_holiday(self):
        """
        2024年祝日
        """
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 1, 1)), '元日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 1, 8)), '成人の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 2, 11)), '建国記念の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 2, 12)), '建国記念の日 振替休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 2, 23)), '天皇誕生日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 3, 20)), '春分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 4, 29)), '昭和の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 5, 3)), '憲法記念日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 5, 4)), 'みどりの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 5, 5)), 'こどもの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 5, 6)), 'こどもの日 振替休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 7, 15)), '海の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 8, 11)), '山の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 8, 12)), '山の日 振替休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 9, 16)), '敬老の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 9, 22)), '秋分の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 9, 23)), '秋分の日 振替休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 10, 14)), 'スポーツの日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 11, 3)), '文化の日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 11, 4)), '文化の日 振替休日')
        self.assertEqual(jpholiday.is_holiday_name(datetime.date(2024, 11, 23)), '勤労感謝の日')

    def test_count_month(self):
        """
        2024年月祝日数
        """
        self.assertEqual(len(jpholiday.month_holidays(2024, 1)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2024, 2)), 3)
        self.assertEqual(len(jpholiday.month_holidays(2024, 3)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2024, 4)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2024, 5)), 4)
        self.assertEqual(len(jpholiday.month_holidays(2024, 6)), 0)
        self.assertEqual(len(jpholiday.month_holidays(2024, 7)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2024, 8)), 2)
        self.assertEqual(len(jpholiday.month_holidays(2024, 9)), 3)
        self.assertEqual(len(jpholiday.month_holidays(2024, 10)), 1)
        self.assertEqual(len(jpholiday.month_holidays(2024, 11)), 3)
        self.assertEqual(len(jpholiday.month_holidays(2024, 12)), 0)

    def test_count_year(self):
        """
        2024年祝日数
        """
        self.assertEqual(len(jpholiday.year_holidays(2024)), 21)



import os
import re
from codecs import open

from setuptools import setup, find_packages

with open(os.path.join('jpholiday', '__init__.py'), 'r', encoding='utf8') as f:
    version = re.compile(
        r".*__version__ = '(.*?)'", re.S).match(f.read()).group(1)

setup(
    name='jpholiday',
    packages=find_packages(include=['jpholiday', 'jpholiday.*']),
    version=version,
    license='MIT License',
    platforms=['POSIX', 'Windows', 'Unix', 'MacOS'],
    description='Pure-Python Japan Public Holiday Generate',
    author='Vatis',
    author_email='vatis@lalcs.com',
    url='https://github.com/Lalcs/jpholiday',
    keywords=['Japan', 'Holiday'],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: Japanese',
        'Operating System :: MacOS',
        'Operating System :: Microsoft',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
    ],
    data_files=[('', ['README.md'])],
    long_description='{}'.format((open('README.md', encoding='utf8').read())),
    long_description_content_type="text/markdown",
)
